@@ -11,6 +11,7 @@ NULL
 #' @slot quote syntax to quote sql query when creating Athena ddl
 #' @name AthenaConnection
 #' @inheritParams methods::show
+#' @importFrom utils modifyList
 NULL
 
 class_cache <- new.env(parent = emptyenv())
@@ -29,48 +30,69 @@ AthenaConnection <- function(aws_access_key_id = NULL,
                              profile_name = NULL, 
                              aws_expiration = NULL,
                              keyboard_interrupt = NULL,
+                             endpoint_override = NULL,
                              ...){
+  kwargs = list(...)
+  sess_kwargs =  c(
+    aws_access_key_id = aws_access_key_id,
+    aws_secret_access_key = aws_secret_access_key,
+    aws_session_token = aws_session_token,
+    region_name = region_name,
+    botocore_session = botocore_session,
+    profile_name = profile_name,
+    .boto_param(kwargs, .SESSION_PASSING_ARGS)
+  )
   tryCatch(
-    boto3 <- boto$Session(aws_access_key_id = aws_access_key_id,
-                          aws_secret_access_key = aws_secret_access_key,
-                          aws_session_token = aws_session_token,
-                          region_name = region_name,
-                          botocore_session = botocore_session,
-                          profile_name = profile_name,
-                          ...),
+    sess <- do.call(boto$Session, sess_kwargs),
     error = function(e) py_error(e)
   )
-  
   # stop connection if region_name is not set in backend or hardcoded
-  if(is.null(boto3$region_name))
-    stop("AWS `region_name` is required to be set. Please set `region` in .config file, ",
-         "`AWS_REGION` in environment variables or `region_name` hard coded in `dbConnect()`.",
-         call. = FALSE)
-    
-  ptr_ll <- list(Athena = boto3$client("athena"),
-                 S3 = boto3$client("s3"),
-                 glue = boto3$client("glue"))
-    
+  if(is.null(sess$region_name))
+    stop(
+      "AWS `region_name` is required to be set. Please set `region` in .config file, ",
+      "`AWS_REGION` in environment variables or `region_name` hard coded in `dbConnect()`.",
+      call. = FALSE
+    )
+  # set up any endpoint url for each aws service: athena, s3, glue
+  endpoints = set_endpoints(endpoint_override)
+  
+  ptr_ll <- list(
+    Athena = do.call(sess$client,
+      c(service_name="athena", modifyList(.boto_param(kwargs, .CLIENT_PASSING_ARGS), list(endpoint_url = endpoints$athena)))
+    ),
+    S3 = do.call(sess$client,
+      c(service_name="s3", modifyList(.boto_param(kwargs, .CLIENT_PASSING_ARGS), list(endpoint_url = endpoints$s3)))
+    ),
+    glue = do.call(sess$client,
+      c(service_name="glue", modifyList(.boto_param(kwargs, .CLIENT_PASSING_ARGS), list(endpoint_url = endpoints$glue)))
+    )
+  )
   if(is.null(s3_staging_dir) && !is.null(work_group)){
-    tryCatch(s3_staging_dir <- ptr_ll$Athena$get_work_group(WorkGroup = work_group)$WorkGroup$Configuration$ResultConfiguration$OutputLocation,
-             error = function(e) py_error(e))
+    tryCatch({
+      s3_staging_dir <- reticulate::py_to_r(ptr_ll$Athena$get_work_group(
+        WorkGroup = work_group
+      )$WorkGroup$Configuration$ResultConfiguration$OutputLocation)
+      }, error = function(e) py_error(e)
+    )
   }
-    
   s3_staging_dir <- s3_staging_dir %||% get_aws_env("AWS_ATHENA_S3_STAGING_DIR")
   
   if(is.null(s3_staging_dir)) {
-    stop("Please set `s3_staging_dir` either in parameter `s3_staging_dir`, environmental varaible `AWS_ATHENA_S3_STAGING_DIR`",
-         "or when work_group is defined in `create_work_group()`", call. = F)
+    stop(
+      "Please set `s3_staging_dir` either in parameter `s3_staging_dir`, environmental varaible `AWS_ATHENA_S3_STAGING_DIR`",
+      "or when work_group is defined in `create_work_group()`", call. = F
+    )
   }
-  
-  info <- list(profile_name = profile_name, s3_staging = s3_staging_dir,
-               dbms.name = schema_name, work_group = work_group %||% "primary",
-               poll_interval = poll_interval, encryption_option = encryption_option,
-               kms_key = kms_key, expiration = aws_expiration,
-               timezone = character(),
-               keyboard_interrupt = keyboard_interrupt,
-               region_name = boto3$region_name)
-  
+  info <- list(
+    profile_name = profile_name, s3_staging = s3_staging_dir,
+    dbms.name = schema_name, work_group = work_group %||% "primary",
+    poll_interval = poll_interval, encryption_option = encryption_option,
+    kms_key = kms_key, expiration = aws_expiration,
+    timezone = character(),
+    keyboard_interrupt = keyboard_interrupt,
+    region_name = sess$region_name,
+    endpoint_override = endpoints
+  )
   res <- new(
     "AthenaConnection",
     ptr = list2env(ptr_ll, parent = emptyenv()),
@@ -530,10 +552,10 @@ setMethod(
     con_error_msg(conn, msg = "Connection already closed.")
     ll <- db_detect(conn, name)
     retry_api_call(
-      output <- conn@ptr$glue$get_table(
+      output <- py_to_r(conn@ptr$glue$get_table(
         DatabaseName = ll[["dbms.name"]],
         Name = ll[["table"]])$Table)
-    
+    )
     col_names = vapply(output$StorageDescriptor$Columns, function(y) y$Name, FUN.VALUE = character(1))
     partitions = vapply(output$PartitionKeys,function(y) y$Name, FUN.VALUE = character(1))
     c(col_names, partitions)
@@ -580,8 +602,11 @@ setMethod(
     ll <- db_detect(conn, name)
 
     for (i in seq_len(athena_option_env$retry)) {
-      resp <- tryCatch(conn@ptr$glue$get_table(DatabaseName = ll[["dbms.name"]], Name = ll[["table"]]), 
-                       error = function(e) retry_error(e))
+      resp <- tryCatch({
+        py_to_r(conn@ptr$glue$get_table(
+          DatabaseName = ll[["dbms.name"]], Name = ll[["table"]]
+        ))
+      }, error = function(e) retry_error(e))
       
       # exponential step back if error and not expected error
       if(inherits(resp, "error") && !grepl(".*table.*not.*found.*", resp, ignore.case = T)){
@@ -592,11 +617,11 @@ setMethod(
         Sys.sleep(backoff_len)
       } else {break}
     }
-    
-    
-    if (inherits(resp, "error") && !grepl(".*table.*not.*found.*", resp, ignore.case = T)) stop(resp)
-    
-    !grepl(".*table.*not.*found.*", resp[1], ignore.case = T)
+    if (inherits(resp, "error") &&
+        !grepl(".*table.*not.*found.*", resp, ignore.case = T)){
+      stop(resp)
+    }
+    return(!grepl(".*table.*not.*found.*", resp[1], ignore.case = T))
 })
 
 #' Remove table from Athena
@@ -660,7 +685,7 @@ setMethod(
       # Get all s3 objects linked to table
       kwargs <- list(Bucket=s3_path[["bucket"]], Prefix=s3_path[["key"]])
       while(is.null(kwargs[["ContinuationToken"]])) {
-        objects <- do.call(conn@ptr$S3$list_objects_v2, kwargs)
+        objects <- py_to_r(do.call(conn@ptr$S3$list_objects_v2, kwargs))
         all_keys <- c(all_keys, lapply(objects$Contents, function(x) list(Key=x$Key)))
         if(identical(objects$NextContinuationToken, kwargs$ContinuationToken) || length(objects$NextContinuationToken) == 0) break
         kwargs[["ContinuationToken"]] <- objects$NextContinuationToken
@@ -697,10 +722,12 @@ setMethod(
     invisible(TRUE)
 })
 
-#' Send query, retrieve results and then clear result set
+#' @title Send query, retrieve results and then clear result set
 #'
 #' @note If the user does not have permission to remove AWS S3 resource from AWS Athena output location, then an AWS warning will be returned.
-#'       It is better use query caching \code{\link{RAthena_options}} so that the warning doesn't repeatedly show.
+#'       For example \code{AccessDenied (HTTP 403). Access Denied}.
+#'       It is better use query caching or optionally prevent clear AWS S3 resource using \code{\link{RAthena_options}}
+#'       so that the warning doesn't repeatedly show.
 #' @name dbGetQuery
 #' @inheritParams DBI::dbGetQuery
 #' @param statistics If set to \code{TRUE} will print out AWS Athena statistics of query.
